// $Id: TWSDemo11AnlProc.cxx 754 2011-05-18 11:04:52Z adamczew $
//-----------------------------------------------------------------------
//       The GSI Online Offline Object Oriented (Go4) Project
//         Experiment Data Processing at EE department, GSI
//-----------------------------------------------------------------------
// Copyright (C) 2000- GSI Helmholtzzentrum fr Schwerionenforschung GmbH
//                     Planckstr. 1, 64291 Darmstadt, Germany
// Contact:            http://go4.gsi.de
//-----------------------------------------------------------------------
// This software can be used under the license agreements as stated
// in Go4License.txt file which is part of the distribution.
//-----------------------------------------------------------------------

#include "TWSDemo11AnlProc.h"

#include <math.h>

#include "TH1.h"
#include "TH2.h"
#include "Riostream.h"
#include "TROOT.h"

#include "TGo4WinCond.h"
#include "TGo4Analysis.h"

#include "TWSDemo11AnlEvent.h"
#include "TWSDemo11UnpackEvent.h"
#include "TWSDemo11Parameter.h"

//-----------------------------------------------------------
TWSDemo11AnlProc::TWSDemo11AnlProc() :
   TGo4EventProcessor(),
   fSum1(0), fSum2(0), fSum3(0),
   fParam1(0), fWinCon(0)
{
}
//-----------------------------------------------------------
TWSDemo11AnlProc::TWSDemo11AnlProc(const char* name) :
   TGo4EventProcessor(name)
{
   cout << "**** TWSDemo11AnlProc: Create" << endl;
   //// init user analysis objects:
   fParam1 = (TWSDemo11Parameter*)  GetParameter("WSDemo11Parameter");
////////////////////////////////////////////////////
// uncomment following lines if you want to use external macro to set parameter values:
//   // if unpack was enabled, parameters have been printed already
//   if(TGo4Analysis::Instance()->GetAnalysisStep("Unpack")->IsProcessEnabled())
//	    gROOT->ProcessLine(".x setparam.C(0)");
//   else gROOT->ProcessLine(".x setparam.C(1)");
///////////////////////////////////////////////////

   if(fParam1->fbHisto){
      // this one is created in TWSDemo11Analysis, because it is used in both steps
      fWinCon = (TGo4WinCond *) GetAnalysisCondition("wincon1");
      if (fWinCon) fWinCon->PrintCondition(true);
      fSum1     = MakeTH1('I', "Sum1", "Sum over 8 channels", 5000, 1., 5001.);
      fSum2     = MakeTH1('I', "Sum2", "Sum over 8 channels shift 1", 5000, 1., 5001.);
      fSum3     = MakeTH1('I', "Sum3", "Sum over 8 channels shift 2", 5000, 1., 5001.);

        

	InitCalibration();
   }
}

void TWSDemo11AnlProc::InitCalibration()
{
  SetMakeWithAutosave(kFALSE);
 for(int i=0;i<WSDemo11_NUM_CHAN;i++) {
	Double_t lo=fParam1->CalibratedEnergy(i,1.);
	Double_t up=fParam1->CalibratedEnergy(i,5001);
         fCalibCr1Ch[i] = MakeTH1('I', Form("Calibrated/Crate1/CalibCr1Ch%02d",i+1), Form("Calibrated Crate 1 channel %2d",i+1), 5000, lo, up);



//          fCalibfCr2Ch[i] = MakeTH1('I', Form("Calibrated/Crate2/CalibCr2Ch%02d",i+1), Form("Crate 2 channel %2d",i+1), 5000, 1., 5001.);
	 
}
SetMakeWithAutosave(kTRUE);
}

//-----------------------------------------------------------
TWSDemo11AnlProc::~TWSDemo11AnlProc()
{
   cout << "**** TWSDemo11AnlProc: Delete" << endl;
   if(fParam1->fbHisto){
      if (fWinCon) fWinCon->PrintCondition(true);
   }
}
//-----------------------------------------------------------
Bool_t TWSDemo11AnlProc::BuildEvent(TGo4EventElement* dest)
{
	Bool_t isValid=kFALSE; // validity of output event

   TWSDemo11UnpackEvent* inp_evt  = (TWSDemo11UnpackEvent*) GetInputEvent();
   TWSDemo11AnlEvent* out_evt = (TWSDemo11AnlEvent*) dest;

   // see comments in UnpackProc
   if((inp_evt==0) || !inp_evt->IsValid()){ // input invalid
	  out_evt->SetValid(isValid); // invalid
	  return isValid; // must be same is for SetValid
   }
   isValid=kTRUE;

     if(fParam1->fbNewcalibration){
	fParam1->fbNewcalibration=kFALSE;
	InitCalibration();
}


   Int_t cnt(0);
   for(Int_t ii=0;ii<WSDemo11_NUM_CHAN/2;ii++) {
      out_evt->frData[cnt] = (Float_t)inp_evt->fiCrate1[ii];
      cnt++;
   }
   for(Int_t ii=0; ii<WSDemo11_NUM_CHAN/2; ii++) {
      out_evt->frData[cnt] = (Float_t)inp_evt->fiCrate2[ii];
      cnt++;
   }
   if(fParam1->fbHisto) { // histogramming
      for(Int_t ii=0;ii<WSDemo11_NUM_CHAN;ii++)
	{
         if(out_evt->frData[ii]) {
            if(fWinCon && fWinCon->Test(out_evt->frData[ii])) fSum1->Fill(out_evt->frData[ii]);
            if (fParam1) fSum2->Fill(out_evt->frData[ii] + fParam1->frP1);
            if (fParam1) fSum3->Fill(out_evt->frData[ii] + fParam1->frP2);
         }
	Double_t val=fParam1->CalibratedEnergy(ii, inp_evt->fiCrate1[ii]);
	fCalibCr1Ch[ii]->Fill(val);

	} 
  }

   // see comments in UnpackProc
   out_evt->SetValid(isValid);
   return isValid;
}
